function [outliers, p_sw1, p_sw2] = rsoutliers(y, X, alpha, draw, header, postfix)
% RSOUTLIERS - Find outliers on given significance level. Version 0.1.16.
%
% Outliers are detected using the externally studentized residual if the degrees of freedom allow it.
% Shapiro-Wilk p-values for the null hypothesis of normal distribution is returned both before and after any
% outlayers are removed.
%
% If outliers are found or the Shapiro-Wilk test indicates a deviation from normality, then the standard behaviour is:
%  1. The residual is plotted against what is expected of the normal distribution in a Q-Q plot.
%  2. The residual is also plotted against table order to reveal potential heteroskedasticity.
%     Both plots are made including and excluding outliers. The title of the last subplot can be set
%     by the user to identify the plot, and the user can also choose to show all or no plots.
%
% The syntax is
% [outliers, p_sw1, p_sw2] = rsoutliers(y, X, [alpha], [draw], [title], [postfix])
%
% Arguments
% y is an N by 1 column vector of data to fit.
% X is an N by a matrix containing the values of the predictors used in the regression, including a possible constant 1 column.
% [alpha] is the limit for which p-values are considered outliers and the residual not normally distributed. If alpha is not provided then 0.05 is assumed.
% [draw] if set to 1 or omitted, draw the pictures that fail the normality test, have outliers, or have less than 3 observations. If set to 0: draw nothing, if set to 2: draw all pictures 
% [title] is a voluntary variable for setting the title of the fourth subplot.
% [postfix] is a voluntary parameter which causes rsoutliers to save the plots as pdf files. [postfix] should be a string which will be appended to the file names of the pdf files.
%
% Return values
% outliers is a column vector of observations deemed to be outliers at the given significance level. If there are to few degrees of freedom, then an empty matrix [] is returned.
% p_sw1 is the Shapiro-Wilk p-value for the null hypothesis of normal distribution before outliers are removed.
% p_sw2 is the same, but after outliers are removed. The Shapiro-Wilk test requires at least three observations, otherwise p=-1 is returned.
%
% Please note that rsoutliers requires the swtest.m file which is downloadable here: 
% https://www.mathworks.com/matlabcentral/fileexchange/13964-shapiro-wilk-and-shapiro-francia-normality-tests/content/swtest.m

% Copyright (C) 2016 Mikael Kurula <mkurula@abo.fi>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
% FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, see <http://www.gnu.org/licenses/>.
%
% Author: Mikael Kurula <mkurula@abo.fi>, 2016


% Check input
if nargin < 2 || nargin > 6,
    error('Wrong number of arguments. Try help rsoutliers for instructions.') ;
end
if size(y,2) > 1,
    error('y has more than one column. Try help rsoutliers for instructions.') ;
end
if size(y,2) > 1,
    error('y has more than one column. Try help rsoutliers for instructions.');
end
if size(X,1) ~= length(y),
    error('The numbers of rows of X and y are unequal.');
end
if cond(X'*X) > 1e10,
    warning('The normal equation is ill-conditioned. If the matrix X does not have linearly dependent columns, try standardizing the factor data.');
end

if nargin < 3,
  alpha=0.05
end

if nargin < 4,
  draw=1
end

if nargin < 5,
  header='YOUR TITLE HERE!';
end


% Standard regression
[N,a]=size(X);
b=X\y;
pred=X*b;
r=y-pred;
SSE=r'*r;

% Check for outliers using externally studentized residual if possible
if N-a >= 1,
  H=X*(X'*X)^-1*X';
  hii=diag(H);
  Ssq=( SSE -r.^2./(1-hii) ) / (N-1-a);
  stud_res=r./sqrt(Ssq.*(1-hii));
  p_outl=2*tcdf(-abs(stud_res),N-1-a);
  outliers=find(p_outl<alpha);
  no_outl=find(p_outl>alpha);
else
  no_outl=1:N;  
  outliers=[];
end

if length(r)>=3                     % SWtest requires 3 observations
  [Hsw, p_sw1] = swtest(r, alpha);
else 
  Hsw=1;
  p_sw1=-1;
end

% Remove outliers
Xn=X(no_outl,:);
yn=y(no_outl);
[~, ~, r_no_outl]=regress(yn,Xn);
if length(r_no_outl)>=3                    
  [~, p_sw2] = swtest(r_no_outl, alpha);
else 
  p_sw2=-1;
end

% Draw pics only if interesting and/or requested by user
if ( (draw==1 & (length(outliers)>0 | Hsw==1) ) | draw==2) & max(r)>min(r)+eps & max(r_no_outl)>min(r_no_outl)+eps
figure
subplot(2,2,1)
normplot(r)
title(['S-W p value: ',num2str(p_sw1)])
subplot(2,2,2)
normplot(r_no_outl)
title([num2str(p_sw2), ' p with outliers removed: ', num2str(outliers')])
subplot(2,2,3)
plot(r,'r*')
title('Residual vs. order')
subplot(2,2,4)
plot(r_no_outl,'r*')
title(header)
if nargin == 6,
  print(['rsoutliers',postfix],'-dpdf')
end

end

end
